/******************************************************************************************/
/*																						  */
/* File Name:  Ovl_Mgr.asm									 						  	  */
/*																						  */	 
/* Date:  August 27, 2002																  */
/*																						  */		
/* Purpose:  This file handles all the actual DMA'ing of the overlays into internal		  */
/* 			 memory.  It first checks to see if the overlay was already DMA'ed in		  */	
/*	 		 previously.  If so, there's no need to re-DMA the same overlay again.		  */
/* 			 It just jumps to the beginning of the overlay internal memory run address	  */
/* 			 to start executing.  On the other hand, if the overlay hasn't been DMA'ed	  */
/*			 in yet, then a DMA setup process starts.  									  */
/*			 																			  */			
/*			 This process first determines whether the overlay to be DMA'ed is overlay 1. */
/*			 If so, then the modifier can be kept at 0 because the overlay information    */
/*			 (live address, size, type) would be at the start the each respective buffer. */
/*			 If the overlay to be DMA'ed isn't overlay 1, then the code checks to find	  */
/* 			 the correct modify value to locate the overlay information (live address,    */
/*			 size, type).																  */	
/* 			 																			  */		
/*			 The next step is to determine whether the overlay to be DMA'ed is a single   */
/*			 section or multiple section overlay.  If it is a single section, then        */
/*			 the DMA parameters are already available because the previous section had    */
/*			 determined the modifier to locate the info in their respective buffers.      */
/*			 However, if this is a multiple section overlay, then each section has to be  */
/*			 DMA'ed separately.  And the code accounts for the number of times to DMA     */
/*			 before the entire overlay is brought in and ready to execute.  It also 	  */
/*			 accounts for sections of nop's, where a zero loop fills instruction memory   */
/*			 with the count of nops rather than DMA. 									  */
/*																						  */
/******************************************************************************************/




#include "def21161.h"

//This data segment sets up the array sizes for the overlay run sizes,
//overlay run starting addresses, live sizes, and live starting address.
//In addition, it defines the buffers for the individual section live 
//addresses, sizes, and types.

.segment/dm seg_dmda;

.var runWordSize[4] = _ov_word_size_run_1,
					  _ov_word_size_run_2,
					  _ov_word_size_run_3,
					  _ov_word_size_run_4;
.global runWordSize;

.var runaddresses[4] = _ov_runtimestartaddress_1,
					   _ov_runtimestartaddress_2,
					   _ov_runtimestartaddress_3,
					   _ov_runtimestartaddress_4;
.global runaddresses;
					   
.var evalbootinfo;
.var num_ovl_sec[4];
.global num_ovl_sec;

.var total_live_addr[10];
.global total_live_addr;

.var total_sec_size[10];
.global total_sec_size;

.var total_sec_type[10];
.global total_sec_type;

.var ovl_in=0;
.global ovl_in;

.var count_sec_size;
.global count_sec_size;


//The overlay manager begins in the following section, and handles all of the tasks mentioned
//in the "Purpose" section of the header above.

.segment/pm mem_pmco;
.global _OverlayManager;

	
_OverlayManager:

	//Initialize M-registers (modify value)
	//Set M2 and M9 to contain the overlay id (R0)
	//Set M4, M8, M11 to 0
	//Set M5 and M10 to 1
	M2=R0;	M4=0;	M5=1;
	M9=R0;	M8=0;  	M10=1;  
			M11=0;
	
	//Not using circular buffer.  Set corresponding L registers to 0.			
	L9 = 0;  L10 = 0;  L11 = 0;  L12 = 0;  L13 = 0;  L14 = 0;	L15=0;
	
	//Initialize index registers
	I8 = ovl_in;				/* pointer to variable containing overlay currently in internal mem */
	I9 = total_live_addr;		/* pointer to buffer containing all the live addresses of overlay sections */
	I10 = total_sec_size;		/* pointer to buffer containing all the overlay sections' sizes */
	I11 = runWordSize -1;		/* pointer to buffer containing each overlay's total run size */ 
	I12 = total_sec_type;		/* pointer to buffer containing all the overlay sections' types */
	I13 = num_ovl_sec - 1;		/* pointer to buffer containing number of sections in each overlay */
	
	//Check ovl_in to see if overlay has already been DMA'ed in
	R2 = PM(I8,M8);
	R3 = R2-R0;
	
	//If equal to 0, then already DMA'ed in.  skip to execution, 
	//do nothing to ovl_in variable
	if eq jump exec_ovl;
	
	//If not DMA'ed in yet, update ovl_id, start DMA routine
	PM(I8,M8) = R0;

	//Before DMA routine can be executed, first find out 
	//1) how many sections are in overlay to be DMA'ed
	//2) whether or not overlay to be DMA'ed is overlay 1
	R2 = PM(M9, I13);			/* read number of sections in overlay to be DMA'ed */
	R8 = 0x0;
	R3 = 0x1;					/* determine if overlay to be DMA'ed is overlay 1 */
	R4 = R0-R3;
	if eq jump here;			/* if subtract equals 0, then overlay to be DMA'ed is overlay 1. */
								/* therefore, jump to the label "here" */
	
	//If overlay is not equal to overlay 1, then find out total number of sections there are before the current overlay */						
	I13 = num_ovl_sec;			/* Initialize I13 back to beginning of buffer containing number of sections in overlay */
jump_here:	R5 = PM(I13, M10);	/* Read from this buffer into R4.  Increment pointer by 1. */
	R8 = R8 + R5;				/* Hold the accumulated count in R8.  */
	R4 = R4 - R3;				/* R4 acts as a loop counter to check whether or not to continue counting the */
								/* total number of sections prior to overlay to be DMA'ed */
	if ne jump jump_here;	
															

here:								
	M11 = R8;					/* R8 contains the value to increment the buffers to the right overlay currently to be */
								/* DMA'ed.  Initialize M11 to contain this modify value */						
	
	//If number of sections in overlay is 1, then DMA setup routine can start.
	//If there's more than one section in the overlay to be DMA'ed, more housekeeping is needed before DMA can start. */
	R4 = R3-R2;
	if eq jump setup_dma;
	
	//Set pointers to point to current overlay in question 
	modify(I9, M11);	/* live addresses of each section */
	modify(I10, M11);	/* sizes of each section */
	modify(I12, M11);	/* types of each section */
	R12 = PM(I10, M8);	/* read count of first section */
	DM(count_sec_size) = R12;	/* keep a running count of section sizes in overlay */
	R14=R12;	
	
	R11 = 0x1;			
		
	R6 = R2;			/* set R6 to contain number of sections in overlay to be DMA'ed */
						/* R6 will act as a loop counter for number of sections to be individually DMA'ed into */
						/* internal memory to make up the entire overlay */

repeat:	R7 = PM(I9, M10);	/* R7 = Live Address */
	R9 = PM(I10, M10); 	/* R9 = Count size */
	R10 = PM(I12, M10);	/* R10 = Type */
	
	//Check type info to see if it's a zero type.
	//If zero type, then go to zero-fill loop.
	//If init type, then continue with setting up DMA.
	
	R4 = R9*R15 (uui);
	R5=R1;
	
	R11 = pass R11;
	if eq call get_index;
	
	R11 = 0;
	R12 = 0x19;
	R12 = R12 - R10;
	if ne jump continue;
	call zero_fill;
	R6 = R6 - R3;
	if ne jump repeat;
	jump exec_ovl;
	
	//If this is a multiple section overlay, then check to see if the first section setup.
	//If it is, there's no need to execute get index.  
	//If it isn't, then need to read DMA internal index pointer and modify according to section size.
get_index:
	
	R5 = R5+R14;					/* holds next index value */
	R14 = R14 + R9;
	rts;
	
	//The "continue" section sets up the DMA for multiple section overlays.
	//It writes the appropriate values to the DMA registers, and calls the "setup_int"
	//routine to enable the DMA, and wait for completion.
	
continue:	
	R12 = 0;
	DM(DMAC10) = R12;    			/* initially disable and clear out DMA channel 10 */ 
	
	DM(EIEP0) = R7;					/* write Live Address to EPB0 external index address register */	

	R12 = 1;						/* DMA int/ext modifys = 1 */
	DM(EMEP0) = R12;				/* setup EPB0 external and internal index modify registers */
	DM(IMEP0) = R12;
	
	R13=DM(SYSCON);         		/* Save old value of SYSCON */
    R12 = BSET R13 BY 1;
	DM(SYSCON)=R12;

	DM(IIEP0) = R5;					/* write Run Address to EPB0 internal index address register */

	DM(CEP0) = R9;					/* set number of "Run" internal words to transfer */

	R9 = R9 * R15(uui);			/* set number of "Live" external words to transfer */
	DM(ECEP0) = R9;	
	
	call setup_int;
	
	R6 = R6 - R3;
	if ne jump repeat;
	jump exec_ovl;

	
	//The "setup_dma" function takes care of DMA'ing a one section overlay.
	//It first checks if the one section is a zero-fill (not likely that entire overlay 
	//but still check for safeguard).  If it is zero section, then call zero-fill routine.
	//Otherwise, setup DMA parameters and enable DMA.
setup_dma:																							
	

	R12 = 0x19;
	R12 = R12 - R10;
	if ne jump continue_setup;
	call zero_fill;
	jump exec_ovl;
	
	//The "continue_setup" section does the same thing as the "continue" section above except
	//it applies to single section overlays.
	//It sets up the DMA using the parameters to DMA only once for single section overlays. 
	//and then calls the "setup_int" routine to enable the DMA, and wait for completion.
	
continue_setup:	
	R12 = 0;
	DM(DMAC10) = R12;    			/* initially disable and clear out DMA channel 10 */ 
	
	R12 = PM(M11,I9);    			/* External Live Address from total_live_addr buffer */
	DM(EIEP0) = R12;				/* write Live Address to EPB0 external index address register */	

	R12 = 1;						/* DMA int/ext modifys = 1 */
	DM(EMEP0) = R12;				/* setup EPB0 external and internal index modify registers */
	DM(IMEP0) = R12;
	
	R13=DM(SYSCON);         		/* Save old value of SYSCON */
    R12 = BSET R13 BY 1;
	DM(SYSCON)=R12;
	
	DM(IIEP0) = R1;				/* write Run Address to EPB0 internal index address register */

    R12 = PM(M11,I10);
	DM(CEP0) = R12;					/* set number of "Run" internal words to transfer */

	R12 = R12 * R15(uui);			/* set number of "Live" external words to transfer */
	DM(ECEP0) = R12;
	
	call setup_int;
	jump exec_ovl;
	
	//The zero_fill routine uses the count value and internal DMA index address to zero-fill
	//the correct amount of zeros to accomodate for nop sections of code.
zero_fill:			
					
	I5 = R5;								/* R5 = internal DMA index pointer */
	PX=0x0;	
	lcntr = r9, do zero_loop until lce;		/* R4 = number of zero fills to write (section count value) */
zero_loop:	DM(I5, M5)=PX;					/* zero fill locations with 0 */
	R12=R5+R9;
	DM(IIEP0)=R12;							/* update internal DMA index pointer with new R5 value */
	R11 = 0;								/* clear count in R11 */
	rts;									/* return from subroutine call */	

setup_int:
	R12=0x421;
	DM(DMAC10) = R12;
	IDLE;
	DM(SYSCON)=R13;
	rts;	
							
	//This function, exec_ovl, goes to the beginning location of overlay in internal memory 
	//to start execution of the DMA'ed overlay.
exec_ovl:
	I15 = R1;					/* places the overlay's beginning run address in index pointer I15 */
	jump (M8, I15);				/* jumps to the pointer location with modify of 0 */
